##############################################################################
#
# Cascading Folders, for Zope 2.3.1
# Copyright (C) 2001 Spoke Technologies
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author: Randall Kern, randy@spoke.net
# Spoke Technologies, Inc
# 677 120th Ave NE #101
# Bellevue, WA 98005
##############################################################################

"""
Cascading Folders
"""

import Globals
from OFS.Folder import Folder
from AccessControl import getSecurityManager


addCascadingFolderForm = Globals.HTMLFile('dtml/addCascadingForm', globals())
def addCascadingFolder(self, id, title='',
                       createPublic=0, createUserF=0,
                       REQUEST=None):
    """
    Add a new Cascading Folder
    """

    ob = CascadingFolder()
    ob.id = str(id)
    ob.title = title

    self._setObject(id, ob)
    ob = self._getOb(id)

    checkPermission = getSecurityManager().checkPermission
    if createUserF:
        if not checkPermission('Add User Folders', ob):
            raise 'Unauthorized', (
                'You are not authorized to add User Folders.'
                )
        ob.manage_addUserFolder()

    if createPublic:
        if not checkPermission('Add Documents, Images, and Files', ob):
            raise 'Unauthorized', (
                'You are not authorized to add DTML Documents.'
                )
        ob.manage_addDTMLDocument(id='index_html', title='')

    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)


_marker=[]
class CascadingFolder(Folder):
    """
    Cascading Folders

    This object is a standard Folder, that will look for objects in an uncle
    if it is not found in this Folder. (The uncle must have the same id.)

    The most common use for this Product is to organize images in your site
    into separate image folders.  For example:

    root
        images (Folder)
            logo.gif (Image)
            nav.gif (Image)
        private (Folder)
            images (Cascading Folder)
                nav.gif (Image)

    Given this site, a request for /private/images/logo.gif would fail if private
    where a normal Zope Folder.  Using a Cascading Folder, a request for logo.gif
    will be passed off to an uncle with the same name, in this case /images.

    Author: Randall Kern, randy@spoke.net
    """

    meta_type = 'Cascading Folder'

    def _getOb(self, id, default=_marker):
        """
        If we don't have an object with the correct id, check in the uncle we're named after.
        """

        ob = Folder._getOb(self, id, _marker)
        if ob is not _marker: return ob

        try:
            grandparent = self.aq_parent.aq_parent
            uncle = grandparent._getOb(self.getId())
            return uncle._getOb(id)
        except:
            if default is _marker: raise AttributeError, id
            return default
